/******************************************************************************/
/*                                                                            */
/*    Copyright (c) Centre de traitement en imagerie virtuelle inc.           */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/*    These coded instructions, statements, and computer programs  contain    */
/*    unpublished  proprietary and  confidential information of  Centre de    */
/*    traitement  en  imagerie virtuelle inc. (CTIV), and are protected by    */
/*    Federal copyright law. They may not be disclosed to third parties or    */
/*    or duplicated in any form, in whole or in  part, without  the  prior    */
/*    written consent of CTIV.                                                */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//     3delight
// = FILENAME
//     dlColor.h
// = AUTHOR(S)
//     Franck Diard
// = VERSION
//     $Revision: 1.1 $
// = DATE RELEASED
//     $Date: 2000/06/21 18:46:36 $
// = RCSID
//     $Id: dlColor.h,v 1.1 2000/06/21 18:46:36 ours Exp $
// = COPYRIGHT
//     Copyright  1999-2000 Centre de traitement en imagerie virtuelle inc.
//     All Rights Reserved.
// ===========================================================================

#ifndef __dlColor_H
#define __dlColor_H

#include <math.h>

typedef float DlRGBColor[3];


extern const DlRGBColor k_DlRGBColorBlack;

#define COLOR_ZERO( C ) ( (C)[0]==0.0 && (C)[1]==0.0 && (C)[2]==0.0 )

#define COLOR_COPY(C1, C2) \
    { \
        (C1)[0] = (C2)[0]; \
        (C1)[1] = (C2)[1]; \
        (C1)[2] = (C2)[2]; \
    }

#define COLOR_SET(C, R ,G, B) \
	{ \
		(C)[0] = (float)R;\
		(C)[1] = (float)G;\
		(C)[2] = (float)B;\
	}

#define COLOR_ADD( CD, C1, C2 ) \
	{ \
		(CD)[0] = (C1)[0] + (C2)[0]; \
		(CD)[1] = (C1)[1] + (C2)[1]; \
		(CD)[2] = (C1)[2] + (C2)[2]; \
	}

#define COLOR_SUB( CD, C1, C2 ) \
	{ \
		(CD)[0] = (C1)[0] - (C2)[0]; \
		(CD)[1] = (C1)[1] - (C2)[1]; \
		(CD)[2] = (C1)[2] - (C2)[2]; \
	}

#define COLOR_ADD_IN_PLACE( CD, C ) \
	{ \
		(CD)[0] += (C)[0]; \
		(CD)[1] += (C)[1]; \
		(CD)[2] += (C)[2]; \
	}

#define COLOR_SCALE( CD, C1, K ) \
	{ \
		(CD)[0] = (C1)[0] * (K); \
		(CD)[1] = (C1)[1] * (K); \
		(CD)[2] = (C1)[2] * (K); \
	}

// CD=c1*k +c2
#define COLOR_SCALE_ADD( CD, C1, C2, K ) \
	{ \
		(CD)[0] = (C2)[0] + (C1)[0] * (K); \
		(CD)[1] = (C2)[1] + (C1)[1] * (K); \
		(CD)[2] = (C2)[2] + (C1)[2] * (K); \
	}

#define COLOR_SCALE_IN_PLACE( CD, K ) \
	{ \
		(CD)[0] *= (K); \
		(CD)[1] *= (K); \
		(CD)[2] *= (K); \
	}

/**
   Multiply a color by a scalar and add it to a color.

   CD = F*C1+C2

   @param DlRGBColor* CD.
   @param DlRGBColor* C1.
   @param DlRGBColor* C2.
   @param DlReal F.
*/
#define COLOR_PROD_ADD(CD, F , C1, C2) \
	{ \
		(CD)[0] = (F)* (C1)[0] + (C2)[0] ;\
		(CD)[1] = (F)* (C1)[1] + (C2)[1] ;\
		(CD)[2] = (F)* (C1)[2] + (C2)[2] ;\
	}

#define COLOR_PROD( CD, C1, C2 ) \
	{ \
		(CD)[0] = (C1)[0] * (C2)[0]; \
		(CD)[1] = (C1)[1] * (C2)[1]; \
		(CD)[2] = (C1)[2] * (C2)[2]; \
	}

#define COLOR_PROD_IN_PLACE( CD, C ) \
	{ \
		(CD)[0] *= (C)[0]; \
		(CD)[1] *= (C)[1]; \
		(CD)[2] *= (C)[2]; \
	}

#define COLOR_MIX( CD, C1, C2, K ) \
	{ \
		(CD)[0] = (K)*(C2)[0] + (1.0f-(K))*(C1)[0]; \
		(CD)[1] = (K)*(C2)[1] + (1.0f-(K))*(C1)[1]; \
		(CD)[2] = (K)*(C2)[2] + (1.0f-(K))*(C1)[2]; \
	}

#define COLOR_GRAY( C )  ( ((C)[0] + (C)[1] + (C)[2])/3.0 )

inline void ColorSetFromHSV(DlRGBColor& o_color, float i_h, float i_s, float i_v)
{
	if (i_s==0)
	{
		COLOR_SET(o_color, i_v, i_v, i_v);
	}
	else
	{ 
		if (i_h == 360.)
		{
			i_h = 0.;
		}
		
		i_h = i_h/60.0f;
		
		int i;
		float f, p, q, t;
		
		i = int(floor(i_h));
		
		f = i_h - i;
		p = i_v * (1.0f - i_s);
		q = i_v * (1.0f - (i_s * f));
		t = i_v * (1.0f - (i_s * (1.0f - f)));
		
		switch (i)
		{ 
		case 0: COLOR_SET(o_color, i_v, t,   p); break ;
		case 1: COLOR_SET(o_color, q,   i_v, p); break ;
		case 2: COLOR_SET(o_color, p,   i_v, t); break ;
		case 3: COLOR_SET(o_color, p,   q,   i_v); break ;
		case 4: COLOR_SET(o_color, t,   p,   i_v); break ;
		case 5: COLOR_SET(o_color, i_v, p,   q); break ;
		}
	}
}

#endif				 // __dlColor_H
