/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/


// ===========================================================================
// = LIBRARY
//     3delight
// = FILENAME
//     dlShaderToken.h
// = AUTHOR(S)
//     Aghiles Kheffache
//     Patrick Fournier
// = VERSION
//     $Revision: 1.10 $
// = DATE RELEASED
//     $Date: 2003/04/16 00:02:23 $
// = RCSID
//     $Id: dlShaderToken.h,v 1.10 2003/04/16 00:02:23 aghiles Exp $
// ===========================================================================

#ifndef __dlShaderToken_H
#define __dlShaderToken_H

#include "dlTokenInfo.h"
#include "dlInterface.h"


class DlToken;

/*
	DlShaderToken
	
	this class is a link between a registry token and a shader variable
	declaration.

	NOTES
	- this is used to transform Ri variables to shader variables;
	- for example, a vertex Ri variable can be transformed to a varying
	shader variable;
	- if m_token == 0x0, then the variable has no declaration in the registry,
	ie, the user did not use "RiDeclare" to register the variable.
*/
class DlShaderToken
{
public:

	DL_INTERFACE DlShaderToken();
	DL_INTERFACE ~DlShaderToken();

	/*
		Init

		inits this object

		PARAMETERS
		i_token   : the resgirty token (could be 0x0);
		i_name    : token name, as declared in the registry;
		i_declaration
		          : the shader variable declaration (such as 'uniform color').
	*/	
	DL_INTERFACE void Init(
		const DlToken* i_token,
		const char *i_name = 0x0,
		const char* i_declaration = 0x0 );

	const DlToken* RegistryToken(void) const { return m_token; }

	/*
		ShaderDeclaration
		
		returns token declaration as specified in the shader
	*/
	const DlTokenInfo& ShaderDeclaration(void) const
	{
		return m_tokenInfo;
	}

	/*
		Declaration
	*/
	const DlTokenInfo& Declaration( void ) const;

	const char *Name( void ) const { return m_name; }

protected:
	/* Token having the same name as this one in the registry */
	const DlToken* m_token;

	/* Token type and class in the shader owning this object */
	DlTokenInfo m_tokenInfo;

	/* Name of the variable, as declared in the shader */
	const char *m_name;
};


#endif // __dlShaderToken_H

