/******************************************************************************/
/*                                                                            */
/*    Copyright (c)The 3Delight Developers.                                   */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//     3delight
// = FILENAME
//     dlSurfaceHidden.h
// = AUTHOR(S)
//     Patrick Fournier
// = VERSION
//     $Revision: 1.14 $
// = DATE RELEASED
//     $Date: 2002/04/02 20:20:50 $
// = RCSID
//     $Id: dlSurfaceHidden.h,v 1.14 2002/04/02 20:20:50 aghiles Exp $
// = COPYRIGHT
//     Copyright  1999-2000 The 3Delight Developers.
//     All Rights Reserved.
// ===========================================================================

#ifndef __dlSurfaceHidden_H
#define __dlSurfaceHidden_H

#include "dlTokenInfo.h"
#include "dlVar.h"
#include "dlBasicTypes.h"

#include "dlHidden.h"

class DlTokenRegWithDefault;

/*
	DlSurfaceHidden
*/
class DlSurfaceHidden : public DlHidden
{
public:
	DlSurfaceHidden();
	~DlSurfaceHidden();

	/* Allocate all DlVars */
	void Init(const DlTokenRegWithDefault* i_reg);

	/* Allocate only DlVars needed by the surface or displacement shader */
	virtual void Init( const DlTokenRegWithDefault*, unsigned i_neededMask );

	DlRGBColor* GetCs(void) const { return m_Cs ? (DlRGBColor*)m_Cs->Value() : 0; }
	DlRGBColor* GetOs(void) const { return m_Os ? (DlRGBColor*)m_Os->Value() : 0; }
	DlVector3* GetI(void) const { return m_I ? (DlVector3*)m_I->Value() : 0; }
	DlRGBColor* GetCi(void) const { return m_Ci ? (DlRGBColor*)m_Ci->Value() : 0; }
	DlRGBColor* GetOi(void) const { return m_Oi ? (DlRGBColor*)m_Oi->Value() : 0; }

	void SetCs( const DlTokenInfo::DlClass& i_valueClass, unsigned i_numValues, void* i_value);
	void SetOs( const DlTokenInfo::DlClass& i_valueClass, unsigned i_numValues, void* i_value);
	void SetI( const DlTokenInfo::DlClass& i_valueClass, unsigned i_numValues, void* i_value);
	void SetCi( const DlTokenInfo::DlClass& i_valueClass, unsigned i_numValues, void* i_value);
	void SetOi( const DlTokenInfo::DlClass& i_valueClass, unsigned i_numValues, void* i_value);

	/*
		GetVar

		see DlHidden::GetVar(...) for more infos
	*/
	DlVar *GetVar( DlHidden::hiddenNames i_name, DlVar ***o_dst=0x0 ) const;

	unsigned Size( void ) const { return m_size; }

	/*
		DownSample
	*/
	DlSurfaceHidden * DownSample( unsigned, unsigned, unsigned, unsigned ) const;

	/*
		public ResampleColors

		resamples Ci & Oi from a previously downsampled (and then shaded) grid

		PARAMETERS
		i_downSampled : the downsampled hidden parameters
		i_uRes, i_vRes: resolution of _this_ grid
		i_srcURes, i_srcVRes
	    	          : resolution of 'i_downSampled' grid
	*/
	void ResampleColors(
		const DlSurfaceHidden *i_downSampled,
		unsigned i_uRes, unsigned i_vRes, 
		unsigned i_srcURes, unsigned i_srcVRes );

	void CompositeVolumeColors(
		float *i_cv, float *i_ov,
		unsigned i_uRes, unsigned i_vRes,
		unsigned i_srcURes, unsigned i_srcVRes );

private:
	/*
		DownSample
	*/
	void DownSample(
		const float *i_data, float *o_data,
		unsigned i_numChannels,
		unsigned i_uRes, unsigned i_vRes,
		unsigned i_dstUres, unsigned i_dstVRes ) const;

private:
	DlVar *m_I;

	DlVar *m_Cs, *m_Os;
	DlVar *m_Ci, *m_Oi;

	/* total size of this structure along with the values. */
	unsigned m_size;
};

#endif
