/******************************************************************************/
/*                                                                            */
/*    Copyright (c) Centre de traitement en imagerie virtuelle inc.           */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/*    These coded instructions, statements, and computer programs  contain    */
/*    unpublished  proprietary and  confidential information of  Centre de    */
/*    traitement  en  imagerie virtuelle inc. (CTIV), and are protected by    */
/*    Federal copyright law. They may not be disclosed to third parties or    */
/*    or duplicated in any form, in whole or in  part, without  the  prior    */
/*    written consent of CTIV.                                                */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//     3delight
// = FILENAME
//     dlToken.h
// = AUTHOR(S)
//     Patrick Fournier
// = VERSION
//     $Revision: 1.1 $
// = DATE RELEASED
//     $Date: 2000/06/22 00:09:24 $
// = RCSID
//     $Id: dlToken.h,v 1.1 2000/06/22 00:09:24 ours Exp $
// = COPYRIGHT
//     Copyright  1999-2000 Centre de traitement en imagerie virtuelle inc.
//     All Rights Reserved.
// ===========================================================================

#ifndef __dlToken_H
#define __dlToken_H

#include "dlTokenInfo.h"

#ifndef __declspec
#ifndef WIN32
#define __declspec(x)
#endif
#endif


// ===========================================================================
//      C L A S S E S
// ===========================================================================

// ===========================================================================
//
//      CLASS DlToken
//
// ===========================================================================

/**
  DlToken.

  \begin{description}

  \item[CLASS TYPE]
  --

  \item[DESCRIPTION]
  Class encapsulating data describing a RenderMan interface token.

  \item[NOTES]
  --

  \end{description}

  @author Patrick Fournier
  @see --
  */

const unsigned int k_maxNameLenght = 128;
const unsigned int k_sentinelValue = 0x4242;

class DlToken
{
	friend class DlTokenRegistry;
	friend class DlRiOptions;
    
public:
    DlToken& operator=(const DlToken& i_token);
    //bool operator==(const DlToken& i_token) const;

    const char* Name(void) const { return m_name; }
    const DlTokenInfo& Declaration(void) const { return m_declaredAs; }

    // Returns 0: no error
    //         1: syntax error in declaration
    int ChangeDeclaration(const char* i_newDecl);
    
    void CheckIntegrity(void) const;

protected:
    /**@name CONSTRUCTION/DESTRUCTION */
    //@{
    /// Default constructor.
    DlToken();

    /// Copy constructor.
    DlToken(const DlToken& i_token);

    /** Constructor.

        Build a token from its declaration.
        Set i_userDeclaration to true if token is not created by a shader.
    */
    DlToken(const char* i_name, const char* i_declaration);

    /// Destructor.
    ~DlToken();

    //@}

protected:
    char m_name[k_maxNameLenght];
public:
    const unsigned int m_sentinel;
protected:
    DlTokenInfo m_declaredAs;   // as seen by RIB or RI API user
};

#endif				 // __dlToken_H
