/******************************************************************************/
/*                                                                            */
/*    Copyright (c) Centre de traitement en imagerie virtuelle inc.           */
/*    All Rights Reserved.                                                    */
/*                                                                            */
/*    These coded instructions, statements, and computer programs  contain    */
/*    unpublished  proprietary and  confidential information of  Centre de    */
/*    traitement  en  imagerie virtuelle inc. (CTIV), and are protected by    */
/*    Federal copyright law. They may not be disclosed to third parties or    */
/*    or duplicated in any form, in whole or in  part, without  the  prior    */
/*    written consent of CTIV.                                                */
/*                                                                            */
/******************************************************************************/

// ===========================================================================
// = LIBRARY
//     3delight
// = FILENAME
//     dlTokenRegistry.h
// = AUTHOR(S)
//     Patrick Fournier
// = VERSION
//     $Revision: 1.1 $
// = DATE RELEASED
//     $Date: 2000/06/22 00:09:24 $
// = RCSID
//     $Id: dlTokenRegistry.h,v 1.1 2000/06/22 00:09:24 ours Exp $
// = COPYRIGHT
//     Copyright  1999-2000 Centre de traitement en imagerie virtuelle inc.
//     All Rights Reserved.
// ===========================================================================

#ifndef __dlTokenRegistry_H
#define __dlTokenRegistry_H

#ifndef WIN32
#include <hash_map.h>
#else
#include "3DeLightSTL.h"
#endif

#ifndef __declspec
#ifndef WIN32
#define __declspec(x)
#endif
#endif

// ===========================================================================
//      C L A S S E S
// ===========================================================================
class DlToken;

// ===========================================================================
//
//      CLASS DlTokenRegistry
//
// ===========================================================================

/**
  DlTokenRegistry.

  \begin{description}

  \item[CLASS TYPE]
  --

  \item[DESCRIPTION]
  This class holds a registry of all declared tokens.

  \item[NOTES]
  --

  \end{description}

  @author Patrick Fournier
  @see --
  */
class DlTokenRegistry
{
	public:

		typedef enum
		{
			e_noErr = 0,
			e_alreadyDeclared,		// token was already declared
            e_badDeclaration    // error in token declaration
		} DlErrorCode;

		/**@name CONSTRUCTION/DESTRUCTION */
		//@{

		/// Constructor.
		DlTokenRegistry();

		/// Destructor.
		virtual ~DlTokenRegistry();

		//@}
		/** Add a token in the registry. If it is already there, return it.
		 */
		__declspec(dllexport) DlErrorCode RegisterToken(const char* i_name, const char* i_declaration, DlToken*& o_token);
		
		/** Find a token in the registry.

		    @return: the token found, or 0 if not found
		*/
		__declspec(dllexport) DlToken* FindToken(const char* i_name) const;

		void CheckIntegrity(void) const;
	
	protected:
		struct EqKey
		{
			bool operator()(const char* k1, const char* k2) const
			{
				return strcmp(k1, k2) == 0;
			}
		};

		typedef hash_map<const char*, DlToken*, hash<const char*>, EqKey> DlHashMap;

		DlHashMap m_registry;
	    int m_numTokens; // For integrity check
};
#endif				 // __dlTokenRegistry_H
